@@ -4,10 +4,26 @@
 #'
 #' @order 1
 #'
-#' @param json One or more \code{character}s of JSON or paths to files containing JSON.
+#' @param json JSON strings, file paths, or raw vectors.
+#'   \itemize{
+#'     \item \code{fparse()}
+#'       \itemize{
+#'         \item \code{character}: One or more JSON strings.
+#'         \item \code{raw}: \code{json} is interpreted as the bytes of a single
+#'                JSON string.
+#'         \item \code{list} Every element must be of type \code{"raw"} and each
+#'               is individually interpreted as the bytes of a single JSON string.
+#'       }
+#'     \item \code{fload()}
+#'       \itemize{
+#'         \item \code{character}: One or more paths to files (local or remote)
+#'               containing JSON.
+#'     }
+#'   }
 #'
-#' @param query String used as a JSON Pointer to identify a specific element within \code{json}.
-#'   \code{character(1L)}, default: \code{""}
+#' @param query If not \code{NULL}, JSON Pointer(s) used to identify and extract
+#'   specific elements within \code{json}. See Details and Examples.
+#'   \code{NULL}, \code{character()}, or \code{list()} of \code{character()}. default: \code{NULL}
 #'
 #' @param empty_array Any R object to return for empty JSON arrays.
 #'   default: \code{NULL}
@@ -18,11 +34,18 @@
 #' @param single_null Any R object to return for single JSON nulls.
 #'   default: \code{NULL}.
 #'
-#' @param error_ok Whether to allow parsing errors.
+#' @param parse_error_ok Whether to allow parsing errors.
 #'   default: \code{FALSE}.
 #'
-#' @param on_error If \code{error_ok} is \code{TRUE}, \code{on_error} is any
-#'   R object to return when parsing errors occur.
+#' @param on_parse_error If \code{parse_error_ok} is \code{TRUE}, \code{on_parse_error} is any
+#'   R object to return when query errors occur.
+#'   default: \code{NULL}.
+#'
+#' @param query_error_ok Whether to allow parsing errors.
+#'   default: \code{FALSE}.
+#'
+#' @param on_query_error If \code{query_error_ok} is \code{TRUE}, \code{on_query_error} is any
+#'   R object to return when query errors occur.
 #'   default: \code{NULL}.
 #'
 #' @param max_simplify_lvl Maximum simplification level.
@@ -47,19 +70,9 @@
 #'   \itemize{
 #'     \item \code{"double"} or \code{0L}: big integers become \code{double}s
 #'     \item \code{"string"} or \code{1L}: big integers become \code{character}s
-#'     \item \code{"integer64"} or \code{2L}: big integers \code{bit64::integer64}s
+#'     \item \code{"integer64"} or \code{2L}: big integers become \code{bit64::integer64}s
 #'   }
 #'
-#' @param verbose Whether to display status messages.
-#'   \code{TRUE} or \code{FALSE}, default: \code{FALSE}
-#'
-#' @param temp_dir Directory path to use for any temporary files.
-#'   \code{character(1L)}, default: \code{tempdir()}
-#'
-#' @param keep_temp_files Whether to remove any temporary files created by
-#'   \code{fload()} from \code{temp_dir}.
-#'   \code{TRUE} or \code{FALSE}, default: \code{TRUE}
-#'
 #'
 #' @details
 #' \itemize{
@@ -79,6 +92,25 @@
 #'           names each returned element using the file's \code{basename()}.
 #'    }
 #'
+#'    \item \code{query}'s goal is to minimize te amount of data that must be
+#'    materialized as R objects (the main performance bottleneck) as well as
+#'    facilitate any post-parse processing.
+#'    \itemize{
+#'      \item To maximize flexibility, there are two approaches to consider when designing \code{query} arguments.
+#'      \itemize{
+#'        \item \code{character} vectors are interpreted as containing queries that
+#'        meant to be applied to all elements of \code{json=}.
+#'        \itemize{
+#'          \item If \code{json=} contains 3 strings and \code{query=} contains
+#'          3 strings, the returned object will be a list of 3 elements (1 for each element
+#'          of \code{json=}), which themselves each contain 3 lists (1 for each element
+#'          of \code{query=}).
+#'        }
+#'        \item \code{list}s of \code{character} vectors are interpreted as containing
+#'        queries meant to be applied  to \code{json} in a zip-like fashion.
+#'      }
+#'    }
+#'
 #' }
 #'
 #' @author Brendan Knapp
@@ -88,6 +120,13 @@
 #' json_string <- '{"a":[[1,null,3.0],["a","b",true],[10000000000,2,3]]}'
 #' fparse(json_string)
 #'
+#' raw_json <- as.raw(
+#'     c(0x22, 0x72, 0x61, 0x77, 0x20, 0x62, 0x79, 0x74, 0x65, 0x73, 0x20, 0x63,
+#'       0x61, 0x6e, 0x20, 0x62, 0x65, 0x63, 0x6f, 0x6d, 0x65, 0x20, 0x4a, 0x53,
+#'       0x4f, 0x4e, 0x20, 0x74, 0x6f, 0x6f, 0x21, 0x22)
+#' )
+#' fparse(raw_json)
+#'
 #' # controlling type-strictness ===============================================
 #' fparse(json_string, type_policy = "numbers")
 #' fparse(json_string, type_policy = "strict")
@@ -113,6 +152,13 @@
 #' )
 #' fparse(json_strings)
 #'
+#' fparse(
+#'     list(
+#'         raw_json1 = as.raw(c(0x74, 0x72, 0x75, 0x65)),
+#'         raw_json2 = as.raw(c(0x66, 0x61, 0x6c, 0x73, 0x65))
+#'     )
+#' )
+#'
 #' # controlling simplification ================================================
 #' fparse(json_strings, max_simplify_lvl = "matrix")
 #' fparse(json_strings, max_simplify_lvl = "vector")
@@ -127,15 +173,16 @@
 #'        single_null = NA_real_)
 #'
 #' # handling invalid JSON and parsing errors ==================================
-#' fparse("junk JSON", error_ok = TRUE)
-#' fparse("junk JSON", error_ok = TRUE, on_error = "can't parse invalid JSON")
+#' fparse("junk JSON", parse_error_ok = TRUE)
+#' fparse("junk JSON", parse_error_ok = TRUE,
+#'        on_parse_error = "can't parse invalid JSON")
 #' fparse(
 #'     c(junk_JSON_1 = "junk JSON 1",
 #'       valid_JSON_1 = '"this is valid JSON"',
 #'       junk_JSON_2 = "junk JSON 2",
 #'       valid_JSON_2 = '"this is also valid JSON"'),
-#'     error_ok = TRUE,
-#'     on_error = NA
+#'     parse_error_ok = TRUE,
+#'     on_parse_error = NA
 #' )
 #'
 #' # querying JSON w/ a JSON Pointer ===========================================
@@ -144,18 +191,8 @@
 #'     "a",
 #'     {
 #'         "b": {
-#'             "c": [
-#'                 [
-#'                     1,
-#'                     2,
-#'                     3
-#'                 ],
-#'                 [
-#'                     4,
-#'                     5,
-#'                     6
-#'                 ]
-#'             ]
+#'             "c": [[1,2,3],
+#'                   [4,5,6]]
 #'         }
 #'     }
 #' ]',
@@ -163,18 +200,9 @@
 #'     "a",
 #'     {
 #'         "b": {
-#'             "c": [
-#'                 [
-#'                     7,
-#'                     8,
-#'                     9
-#'                 ],
-#'                 [
-#'                     10,
-#'                     11,
-#'                     12
-#'                 ]
-#'             ]
+#'             "c": [[7,8,9],
+#'                   [10,11,12]],
+#'            "d": [1,2,3,4]
 #'         }
 #'     }
 #' ]')
@@ -184,31 +212,54 @@
 #' fparse(json_to_query, query = "1/b/c/1")
 #' fparse(json_to_query, query = "1/b/c/1/0")
 #'
+#' # handling invalid queries ==================================================
+#' fparse(json_to_query, query = "1/b/d",
+#'        query_error_ok = TRUE,
+#'        on_query_error = "d isn't a key here!")
+#'
+#' # multiple queries applied to EVERY element =================================
+#' fparse(json_to_query, query = c(query1 = "1/b/c/1/0",
+#'                                 query2 = "1/b/c/1/1",
+#'                                 query3 = "1/b/c/1/2"))
+#'
+#' # multiple queries applied to EACH element ==================================
+#' fparse(json_to_query,
+#'        query = list(queries_for_json1 = c(c1 = "1/b/c/1/0",
+#'                                           c2 = "1/b/c/1/1"),
+#'                     queries_for_json2 = c(d1 = "1/b/d/1",
+#'                                           d2 = "1/b/d/2")))
+#'
 #' @export
 fparse <- function(json,
-                   query = "",
+                   query = NULL,
                    empty_array = NULL,
                    empty_object = NULL,
                    single_null = NULL,
-                   error_ok = FALSE,
-                   on_error = NULL,
+                   parse_error_ok = FALSE,
+                   on_parse_error = NULL,
+                   query_error_ok = FALSE,
+                   on_query_error = NULL,
                    max_simplify_lvl = c("data_frame", "matrix", "vector", "list"),
                    type_policy = c("anything_goes", "numbers", "strict"),
                    int64_policy = c("double", "string", "integer64")) {
     # validate arguments =======================================================
     # types --------------------------------------------------------------------
-    if (!is.character(json)) {
-        stop("`json=` must be a `character`.")
+    if (!.is_valid_json_arg(json)) {
+        stop("`json=` must be a non-empty character vector, raw vector, or a list containing raw vectors.")
+    }
+    if (!.is_valid_query_arg(query)) {
+        stop("`query=` must be `NULL`, a non-empty character vector, or a list containing non-empty character vectors.")
     }
 
-    if (is.null(query)) {
-        query <- ""
-    } else if (!.is_scalar_chr(query)) {
-        stop("`query=` must be a single, non-`NA` `character`.")
+    if (is.list(query) && length(json) != length(query)) {
+        stop("`query=` is a list (nested query), but is not the same length as `json=`.")
     }
 
-    if (!.is_scalar_lgl(error_ok)) {
-        stop("`error_ok=` must be either `TRUE` or `FALSE`.")
+    if (!.is_scalar_lgl(parse_error_ok)) {
+        stop("`parse_error_ok=` must be either `TRUE` or `FALSE`.")
+    }
+    if (!.is_scalar_lgl(query_error_ok)) {
+        stop("`query_error_ok=` must be either `TRUE` or `FALSE`.")
     }
     # prep options =============================================================
     # max_simplify_lvl ---------------------------------------------------------
@@ -257,21 +308,23 @@ fparse <- function(json,
         stop("`int64_policy` must be of type `character` or `numeric`.")
     }
 
-    if (int64_policy == 2L &&
-        !requireNamespace("bit64", quietly = TRUE)) {
+    if (int64_policy == 2L && !requireNamespace("bit64", quietly = TRUE)) {
         # nocov start
         stop('`int64_policy="integer64", but the {bit64} package is not installed.')
         # nocov end
     }
+
     # deserialize ==============================================================
     .deserialize_json(
         json = json,
-        json_pointer = query,
+        query = query,
         empty_array = empty_array,
         empty_object = empty_object,
         single_null = single_null,
-        error_ok = error_ok,
-        on_error = on_error,
+        parse_error_ok = parse_error_ok,
+        on_parse_error = on_parse_error,
+        query_error_ok = query_error_ok,
+        on_query_error = on_query_error,
         simplify_to = max_simplify_lvl,
         type_policy = type_policy,
         int64_r_type = int64_policy
